# -*- mode: python -*-
# -*- coding: utf-8 -*-

##############################################################################
#
# Gestion scolarite IUT
#
# Copyright (c) 1999 - 2021 Emmanuel Viennet.  All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Emmanuel Viennet      emmanuel.viennet@viennet.net
#
##############################################################################

"""Gestion des images logos (nouveau ScoDoc 9)

Les logos sont `logo_header.<ext>`  et `logo_footer.<ext>`
avec `ext` membre de LOGOS_IMAGES_ALLOWED_TYPES (= jpg, png)

SCODOC_LOGOS_DIR   /opt/scodoc-data/config/logos
"""
import imghdr
import os
import re
from pathlib import Path

from flask import abort, current_app, url_for
from werkzeug.utils import secure_filename

from app import Departement, ScoValueError
from app.scodoc import sco_utils as scu
import PIL
from PIL import Image as PILImage

GLOBAL = "_SERVER"  # category for server level logos
LOGOS_DIR_PREFIX = "logos_"
LOGO_FILE_PREFIX = "logo_"
ALLOWED_EXT = "|".join(scu.LOGOS_IMAGES_ALLOWED_TYPES)
FILENAME_PARSER = re.compile(r"logo_([^.]*).(%s)" % ALLOWED_EXT)


def get_logo(logoname, dept_id=None):
    """Recherche le logo 'name' d'abord dans le département puis si non trouvé au niveau global"""
    logo = Logo(logoname, dept_id)
    try:
        logo.read()
    except ScoValueError:  # logo non trouvé au niveau du département recherche au niveau global
        logo = Logo(logoname=logoname, dept_id=None).read()
    return logo


def get_logo_filename(name, dept_id=None):
    breakpoint()
    return get_logo(name, dept_id).read().filename


def get_logo_url(name, dept_id):
    return get_logo(name, dept_id).read().get_url()


def write_logo(stream, name, dept_id=None):
    Logo(logoname=name, dept_id=dept_id).create(stream)


def list_logos():
    inventory = {GLOBAL: _list_dept_logos()}  # logos globaux (header / footer)
    for dept in Departement.query.filter_by(visible=True).all():
        logos_dept = _list_dept_logos(dept_id=dept.id)
        if logos_dept:
            inventory[dept.acronym] = _list_dept_logos(dept.id)
    return inventory


def _list_dept_logos(dept_id=None):
    logos = {}
    path_dir = Path(scu.SCODOC_LOGOS_DIR)
    if dept_id:
        path_dir = Path(
            os.path.sep.join([scu.SCODOC_LOGOS_DIR, LOGOS_DIR_PREFIX + str(dept_id)])
        )
    if path_dir.exists():
        for entry in path_dir.iterdir():
            if os.access(path_dir.joinpath(entry).absolute(), os.R_OK):
                result = FILENAME_PARSER.match(entry.name)
                if result:
                    logoname = result.group(1)
                    logos[logoname] = Logo(logoname=logoname, dept_id=dept_id).read()
    return logos if len(logos.keys()) > 0 else None


class Logo:
    """Responsable des opérations (read, create)
    du calcul des chemins
    et de la récupération des informations sur un logp
    """

    def __init__(self, logoname, dept_id=None):
        """Initialisation des noms et département des logos.
        Le format est renseigné au moment de la lecture ou de la création du logo
        """
        self.logoname = logoname
        self.scodoc_dept = dept_id
        self.suffix = None
        self.dimensions = None
        if self.scodoc_dept:
            self.dirname = os.path.sep.join(
                [scu.SCODOC_LOGOS_DIR, LOGOS_DIR_PREFIX + secure_filename(str(dept_id))]
            )
        else:
            self.dirname = scu.SCODOC_LOGOS_DIR
        self.basename = os.path.sep.join(
            [self.dirname, LOGO_FILE_PREFIX + secure_filename(self.logoname)]
        )
        self.filename = None

    def set_format(self, fmt):
        self.suffix = fmt
        self.filename = self.basename + "." + fmt

    def _ensure_directory_exists(self):
        "create enclosing directory if necessary"
        if not Path(self.dirname).exists():
            current_app.logger.info(f"sco_logos creating directory %s", self.dirname)
            os.mkdir(self.dirname)

    def create(self, stream):
        img_type = guess_image_type(stream)
        if img_type not in scu.LOGOS_IMAGES_ALLOWED_TYPES:
            abort(400, "type d'image invalide")
        self.set_format(img_type)
        self._ensure_directory_exists()
        filename = self.basename + "." + self.suffix
        with open(filename, "wb") as f:
            f.write(stream.read())
        current_app.logger.info(f"sco_logos.store_image %s", self.filename)
        # erase other formats if they exists
        for suffix in set(scu.LOGOS_IMAGES_ALLOWED_TYPES) - set([img_type]):
            try:
                os.unlink(self.basename + "." + suffix)
            except IOError:
                pass

    def read(self):
        """
        Récupération des données pour un logo existant (sinon -> Exception)
        il doit exister un et un seul fichier image parmi les types autorisés
        (sinon on considère le premier trouvé)
        permet d'affiner le format d'un logo de format inconnu
        """
        for suffix in scu.LOGOS_IMAGES_ALLOWED_TYPES:
            path = Path(self.basename + "." + suffix)
            if path.exists():
                self.set_format(suffix)
                with open(self.filename, "rb") as f:
                    img = PILImage.open(f)
                    self.dimensions = img.size
                return self
        # if no file found, raise exception
        raise ScoValueError(
            "Logo %s not found for dept %s" % (self.logoname, self.scodoc_dept)
        )

    def get_url(self):
        if self.scodoc_dept
        return url_for(
            "scodoc.logo_custom", scodoc_dept=self.scodoc_dept, name=self.logoname
        )


# def get_logo_filename(logo_type: str, scodoc_dept: str) -> str:
#     """return full filename for this logo, or "" if not found
#     an existing file with extension.
#         logo_type: "header" or "footer"
#         scodoc-dept: acronym
#     """
#     # Search logos in dept specific dir (/opt/scodoc-data/config/logos/logos_<dept>),
#     # then in config dir /opt/scodoc-data/config/logos/
#     for image_dir in (
#         scu.SCODOC_LOGOS_DIR + "/logos_" + scodoc_dept,
#         scu.SCODOC_LOGOS_DIR,  # global logos
#     ):
#         for suffix in scu.LOGOS_IMAGES_ALLOWED_TYPES:
#             filename = os.path.join(image_dir, f"logo_{logo_type}.{suffix}")
#             if os.path.isfile(filename) and os.access(filename, os.R_OK):
#                 return filename
#
#     return ""


def guess_image_type(stream) -> str:
    "guess image type from header in stream"
    header = stream.read(512)
    stream.seek(0)
    fmt = imghdr.what(None, header)
    if not fmt:
        return None
    return fmt if fmt != "jpeg" else "jpg"
